/*********************************************************************/
/* PROGRAMA PARA EL PROYECTO DEL SLIDER MOTORIZADO                   */
/* MÁS INFORMACIÓN EN https://fp40thinkandshare.es/descripcion/      */
/*********************************************************************/

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <WiFiClient.h>
#include <AccelStepper.h>

//VALORES CONFIGURABLES EN EL PROGRAMA
/********************************************************************************************************/
const char seg_auto = 30;   // Tiempo en segundos antes de entrar en modo automático
const char password[] = "12345678";   //Definimos la contraseña de nuestro SLIDER
/********************************************************************************************************/

const char ssid[] = "Slider";         //Definimos la SSDI de nuestro servidor WiFi -nombre de red-
const char canal_SSID = 6;          //Definimos el canal donde vamos a emitir
const char max_connection = 1;       //Número máximo de clientes que se pueden conectar
const boolean hidden = false;         //Definimos que el SSID no se ocultará

ESP8266WebServer server(80);

const char bump_izq = 12; //Final de carrera izquierdo del slider
const char bump_der = 13; //Final de carrera derecho del slider
const char pulsado = 1; //Valor cuando se pulsa el final de carrera

const char paso = 4;              //PIN ESP8266 DIRECCIÓN
const char direccion = 5;         //PIN ESP8266 PASO
AccelStepper stepper(AccelStepper::DRIVER, paso, direccion);

//VALORES QUE HACEN QUE EL SLIDER FUNCIONE EN LOS SENTIDOS CORRECTOS
//SI EL SLIDER FUNCIONA AL CONTRARIO INTERCAMBIAR LOS VALORES DE LAS
//VARIABLES DE DERECHA E IZQUIERDA
const int derecha = -1;
const int izquierda = 1;
const int parado = 0;

int spdmax = 200;             // Velocidad máxima del motor
int spdmin = 100;             // Velocidad mínima del motor
float spd = float(spdmax + spdmin) / 2.0; // Velocidad actual y primera velocidad es la mitad entre la máx. y la min
int veltotal = 50;                        // Velocidad inicial del Slider de la aplicación
int sign = 0;                 // Valores: 1 avanzar, 0 parar o -1 retroceder

// Empezamos siempre parado
int signpast = 0;                 // Estado anterior

const char encender = 0;
const char apagar = 1;

int veltotalant;

// VARIABLES MODO AUTOMÁTICO
unsigned long ini_time;     //Variable para controlar el modo automático
int time_auto;              //Variable para controlar el modo automático

void setup()
{
  pinMode(bump_izq, INPUT);  // Bumper izquierda
  pinMode(bump_der, INPUT);  // Bumper derecha

  veltotal = spd;

  delay(1000);
  Serial.begin(115200);           //Configuración de la velocidad del Serial

  // Configuración del ESP8266 como un Access Point
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, password, canal_SSID, hidden, max_connection);
  /*
    ssid - cadena de caracteres que contiene el SSID de la red (máximo 63 caracteres)
    password - cadena de caracteres opcional con una contraseña. Para la red WPA2-PSK, debe tener al menos 8 caracteres de longitud. Si no se especifica, el punto de acceso estará abierto para que cualquiera pueda conectarse.
    channel - parámetro opcional para establecer el canal de WiFi, del 1 aa 13. Canal predeterminado = 1.
    hidden - parámetro opcional, si se establece en true se ocultará el SSID
    max_connection - parámetro opcional para establecer el número máximo de estaciones conectadas simultáneamente, de 0 a 8. Por defecto 4. Una vez que el número máximo se ha alcanzado, cualquier otra estación que quiera conectarse se verá forzada a esperar hasta que alguna estación conectada se desconecte.
  */

  Serial.println();
  Serial.print("Direccion IP Access Point - por defecto: ");      //Imprime la dirección IP
  Serial.println(WiFi.softAPIP());
  Serial.print("Direccion MAC Access Point: ");                   //Imprime la dirección MAC
  Serial.println(WiFi.softAPmacAddress());


  server.on("/", handleRootPath);
  server.on("/Init", handleInit);

  server.begin();
  Serial.println("HTTP server encendido");

  //INICIALIZAMOS EL MOTOR
  spd = map(veltotal, 0, 100, spdmin, spdmax);  //Asignamos la velocidad inicial del slider de la aplicación
  veltotalant = veltotal;                       //Inicializamos la velocidad anterior con la inicial
  stepper.setMaxSpeed(spdmax);                  //Asignamos la velocidad máxima que puede alcanzar el motor

  ini_time = millis();
  //Calculamos del tiempo antes de entrar en modo automático
  time_auto = seg_auto * 1000;
}

boolean automatico = LOW;
void loop()
{
  // SE ACTIVA EL MODO AUTOMÁTICO
  if ((millis() - ini_time > time_auto) && (WiFi.softAPgetStationNum() == 0) && (automatico == LOW))
  {
    sign = 1;
    automatico = HIGH;
  }
  //SE CONECTA UN USUARIO AL SLIDER Y COMIENZA A ESCUCHAR
  if (WiFi.softAPgetStationNum() == 1)
  {
    server.handleClient();
    automatico = HIGH;
  }
  //NOS MUESTRA QUE NO HAY NADIE CONECTADO
  else if (WiFi.softAPgetStationNum() == 0)
  {
    Serial.println("NO HAY NADIE CONECTADO");
  }
  // CAMBIO DE GIRO A DERECHAS POR LLEGAR AL FINAL DE CARRERA DE LA IZQUIERDA
  if ((digitalRead(bump_izq) == pulsado) && (sign != parado))
  {
    signpast = sign;
    sign = derecha;
  }
  else  {}
  // CAMBIO DE GIRO A IZQUIERDAS POR LLEGAR AL FINAL DE CARRERA DE LA DERECHA
  if ((digitalRead(bump_der) == pulsado) && (sign != parado))
  {
    signpast = sign;
    sign = izquierda;
  }
  else {} //else del bumper derecho
  //ACTUALIZAMOS LA VELOCIDAD Y EL SENTIDO DE GIRO Y ENVIAMOS AL MOTOR
  stepper.setSpeed(sign * spd);
  stepper.runSpeed();
}

void handleRootPath() {
  server.send(200, "text/plain", "Funcionando.");
}

boolean estado = HIGH;
//VARIABLE QUE ME DICE SI EL SLIDER SE ENCENDIÓ POR PRIMERA VEZ
//    LOW --> no se ha encendido ninguna vez
//    HIGH --> toma este valor cuando se enciende la primera vez
boolean primera = LOW;
//VARIABLE QUE ME DICE SI EL SLIDER ESTA PARADO PORQUE SE HA MOVIDO DE MANERA MANUAL
//    LOW --> no ha pasado por manual
//    HIGH --> ha entrado en el bucle manual
boolean manual = LOW;

void handleInit()
{
  // Imprime el número de clientes conectados
  Serial.printf("Clientes conectados al Access Point: %d\n", WiFi.softAPgetStationNum());
  /************************************/
  /*RECEPCIÓN DE DATOS DE LA APLICACIÓN*/
  /************************************/
  //RECIBIMOS ORDEN DEL BOTÓN ENCENDER DE LA APLICACIÓN
  if (server.hasArg("ENCENDER"))
  {
    estado = LOW;
    manual = LOW;
    if (primera == LOW)
    {
      //PRIMERA VEZ QUE SE ENCIENDE EL SLIDER
      signpast = sign;
      sign = derecha;
      primera = HIGH;
    }
    else
    {
      if ((sign == parado))
      {
        // SI LO TENÍA PARADO LO VUELVO A ENCENDER
        sign = signpast;
        signpast = sign;
      }
      else
      {
        // YA ESTÁ ENCENDIDO
      }
    }
  }
  //RECIBIMOS ORDEN DEL BOTÓN APAGAR DE LA APLICACIÓN
  if (server.hasArg("APAGAR"))
  {
    estado = HIGH;
    if (sign != parado)
    {
      signpast = sign;
      sign = parado;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE CAMBIO DE VELOCIDAD DE LA APLICACIÓN
  if (server.hasArg("VELOCIDAD"))
  {
    veltotal = (server.arg("VELOCIDAD")).toInt();
    estado = LOW;
    if (veltotal == 0)
    {
      spd = 0;
    }
    else
    {
      // ACTUALIZAMOS LA VELOCIDAD PORQUE HA CAMBIADO
      if (veltotal != veltotalant)
      {
        spd = map(veltotal, 0, 100, spdmin, spdmax);
        veltotalant = veltotal;
      }
      else {}
    }
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE CAMBIO DE GIRO DE LA APLICACIÓN
  if (server.hasArg("GIRAR"))
  {
    if (estado == HIGH)
    {
      estado = LOW;
    }
    else
    {
      estado = HIGH;
    }
    if (sign != parado)
    {
      signpast = sign;
      sign = sign * -1;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE CAMBIO DE GIRO A IZQUIERDA DE LA APLICACIÓN
  if (server.hasArg("IZQUIERDA"))
  {
    // VIENE DE PONERLO EN POSICIÓN MANUAL Y LO PONEMOS EN DIRECCIÓN IZQUIERDA
    if (manual == HIGH)
    {
      manual = LOW;
      signpast = sign;
      sign = izquierda;
    }
    // LO PONEMOS EN DIRECCIÓN IZQUIERDA
    else if (sign == derecha)
    {
      signpast = sign;
      sign = izquierda;
    }
    //YA VA EN DIRECCIÓN IZQUIERDA Y NO HAGO NADA
    else if (sign == izquierda)
    {
    }
    // EL MOTOR ESTÁ PARADO Y LO PONEMOS EN MARCHA EN DIRECCIÓN IZQUIERDA
    else if (sign == parado)
    {
      signpast = sign;
      sign = izquierda;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE CAMBIO DE GIRO A DERECHA DE LA APLICACIÓN
  if (server.hasArg("DERECHA"))
  {
    // VIENE DE PONERLO EN POSICIÓN MANUAL Y LO PONEMOS EN DIRECCIÓN DERECHA
    if (manual == HIGH)
    {
      manual = LOW;
      signpast = sign;
      sign = derecha;
    }
    // LO PONEMOS EN DIRECCIÓN DERECHA
    if (sign == izquierda)
    {
      signpast = sign;
      sign = derecha;
    }
    //YA VA EN DIRECCIÓN DERECHA Y NO HAGO NADA
    else if (sign == derecha)
    {
    }
    // EL MOTOR ESTÁ PARADO Y LO PONEMOS EN MARCHA EN DIRECCIÓN DERECHA
    else if (sign == parado)
    {
      signpast = sign;
      sign = derecha;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE DESPLAZAMIENTO A IZQUIERDA DE MANERA MANUAL
  if (server.hasArg("ON_IZQ"))
  {
    sign = izquierda;
    if (digitalRead(bump_izq) == pulsado)
    {
      signpast = sign;
      sign = parado;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE PARAR EL DESPLAZAMIENTO A IZQUIERDA DE MANERA MANUAL
  if (server.hasArg("STOP_IZQ"))
  {
    sign = parado;
    manual = HIGH;
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE DESPLAZAMIENTO A DERECHA DE MANERA MANUAL
  if (server.hasArg("ON_DER"))
  {
    sign = derecha;
    if (digitalRead(bump_der) == pulsado)
    {
      signpast = sign;
      sign = parado;
    }
    else {}
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE PARAR EL DESPLAZAMIENTO A DERECHA DE MANERA MANUAL
  if (server.hasArg("STOP_DER"))
  {
    sign = parado;
    manual = HIGH;
  }
  //RECIBIMOS ORDEN DEL BOTÓN DE PARAR EL SLIDER
  if (server.hasArg("STOP_SLIDER"))
  {
    sign = parado;
  }
  //MANDAMOS CONTESTACIÓN DE QUE HEMOS RECIBIDO EL MENSAJE DE MANERA CORRECTA Y CERRAMOS LAS CONEXIÓN
  server.send(200, "text/plain", "HTTP/1.1 200 OK\nContent-Type: text/html\nConnection: close\n\n");
}
